@@ -27,6 +27,7 @@
 #undef EXPANDED_GDMCONFIGDIR
 #undef EXPANDED_LOCALEDIR
 #undef EXPANDED_AUTHDIR
+#undef EXPANDED_LOGDIR
 #undef EXPANDED_SYSCONFDIR
 #undef EXPANDED_SESSDIR
 #undef X_SERVER
@@ -35,3 +36,4 @@
 #undef GETTEXT_PACKAGE
 #undef HAVE_LOGINCAP
 #undef HAVE_SYS_SOCKIO_H
+#undef HAVE_SCHED_YIELD
