@@ -22,10 +22,14 @@
 #include <limits.h>
 #include <stdlib.h>
 #include <unistd.h>
+#if defined(_POSIX_PRIORITY_SCHEDULING) && defined(HAVE_SCHED_YIELD)
+#include <sched.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/socket.h>
+#include <sys/resource.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
@@ -77,12 +81,17 @@ gid_t GdmGroupId;		/* Groupid under which gdm should run */
 pid_t extra_process = 0;	/* An extra process.  Used for quickie 
 				   processes, so that they also get whacked */
 int extra_status = 0;		/* Last status from the last extra process */
-pid_t gdm_main_pid = 0;
+pid_t gdm_main_pid = 0;		/* PID of the main daemon */
 gboolean preserve_ld_vars = FALSE; /* Preserve the ld environment variables */
 gboolean no_daemon = FALSE;	/* Do not daemonize */
+gboolean no_console = FALSE;	/* There are no local servers, this means,
+				   don't run local servers and second,
+				   don't display info on the console */
 
 GdmConnection *fifoconn = NULL; /* Fifo connection */
+GdmConnection *pipeconn = NULL; /* slavepipe (handled just like Fifo for compatibility) connection */
 GdmConnection *unixconn = NULL; /* UNIX Socket connection */
+int slave_fifo_pipe_fd = -1; /* the slavepipe connection */
 
 char *gdm_charset = NULL;
 
@@ -99,17 +108,15 @@ gboolean gdm_emergency_server = FALSE;
 gboolean gdm_first_login = TRUE;
 
 int gdm_in_signal = 0;
+gboolean gdm_in_final_cleanup = FALSE;
 
 /* Configuration options */
 gchar *GdmUser = NULL;
 gchar *GdmGroup = NULL;
+gchar *GdmGtkRC = NULL;
 gchar *GdmSessDir = NULL;
 gchar *GdmXsession = NULL;
 gchar *GdmLocaleFile = NULL;
-#if 0
-/* FIXME: Maybe just whack this */
-gchar *GdmGnomeDefaultSession = NULL;
-#endif
 gchar *GdmAutomaticLogin = NULL;
 gboolean GdmAutomaticLoginEnable = FALSE;
 gboolean GdmAlwaysRestartServer = FALSE;
@@ -149,7 +156,6 @@ gchar *GdmDefaultPath = NULL;
 gchar *GdmRootPath = NULL;
 gboolean  GdmKillInitClients = FALSE;
 gint  GdmUserMaxFile = 0;
-gint  GdmSessionMaxFile = 0;
 gboolean  GdmXdmcp = FALSE;
 gint  GdmDispPerHost = 0;
 gint  GdmMaxPending = 0;
@@ -214,6 +220,58 @@ compare_displays (gconstpointer a, gconstpointer b)
 		return 0;
 }	
 
+static void
+check_servauthdir (struct stat *statbuf)
+{
+    int r;
+
+    /* Enter paranoia mode */
+    IGNORE_EINTR (r = stat (GdmServAuthDir, statbuf));
+    if (r < 0) {
+	    char *s = g_strdup_printf
+		    (_("Server Authorization directory "
+		       "(daemon/ServAuthDir) is set to %s "
+		       "but this does not exist. Please "
+		       "correct gdm configuration %s and "
+		       "restart gdm."), GdmServAuthDir,
+		     GDM_CONFIG_FILE);
+	    if ( ! no_console)
+		    gdm_text_message_dialog (s);
+	    GdmPidFile = NULL;
+	    gdm_fail (_("%s: Authdir %s does not exist. Aborting."), "gdm_config_parse", GdmServAuthDir);
+    }
+
+    if (! S_ISDIR (statbuf->st_mode)) {
+	    char *s = g_strdup_printf
+		    (_("Server Authorization directory "
+		       "(daemon/ServAuthDir) is set to %s "
+		       "but this is not a directory. Please "
+		       "correct gdm configuration %s and "
+		       "restart gdm."), GdmServAuthDir,
+		     GDM_CONFIG_FILE);
+	    if ( ! no_console)
+		    gdm_text_message_dialog (s);
+	    GdmPidFile = NULL;
+	    gdm_fail (_("%s: Authdir %s is not a directory. Aborting."), "gdm_config_parse", GdmServAuthDir);
+    }
+}
+
+static void
+check_logdir (void)
+{
+	struct stat statbuf;
+	int r;
+   
+	IGNORE_EINTR (r = stat (GdmLogDir, &statbuf));
+	if (r < 0 ||
+	    ! S_ISDIR (statbuf.st_mode))  {
+		gdm_error (_("%s: Logdir %s does not exist or isn't a directory.  Using ServAuthDir %s."), "gdm_config_parse",
+			   GdmLogDir, GdmServAuthDir);
+		g_free (GdmLogDir);
+		GdmLogDir = g_strdup (GdmServAuthDir);
+	}
+}
+
 /**
  * gdm_config_parse:
  *
@@ -229,11 +287,13 @@ gdm_config_parse (void)
     gchar *bin;
     VeConfig *cfg;
     GList *list, *li;
+    int r;
     
     displays = NULL;
     high_display_num = 0;
 
-    if (stat (GDM_CONFIG_FILE, &statbuf) == -1) {
+    IGNORE_EINTR (r = stat (GDM_CONFIG_FILE, &statbuf));
+    if (r < 0) {
 	    gdm_error (_("%s: No configuration file: %s. Using defaults."),
 		       "gdm_config_parse", GDM_CONFIG_FILE);
     } else {
@@ -288,15 +348,13 @@ gdm_config_parse (void)
     }
     GdmSuspend = ve_config_get_string (cfg, GDM_KEY_SUSPEND);
     GdmLocaleFile = ve_config_get_string (cfg, GDM_KEY_LOCFILE);
-#if 0
-/* FIXME: Maybe just whack this */
-    GdmGnomeDefaultSession = ve_config_get_string (cfg, GDM_KEY_GNOMEDEFAULTSESSION);
-#endif
     GdmUser = ve_config_get_string (cfg, GDM_KEY_USER);
     GdmUserAuthDir = ve_config_get_string (cfg, GDM_KEY_UAUTHDIR);
     GdmUserAuthFile = ve_config_get_string (cfg, GDM_KEY_UAUTHFILE);
     GdmUserAuthFB = ve_config_get_string (cfg, GDM_KEY_UAUTHFB);
 
+    GdmGtkRC = ve_config_get_string (cfg, GDM_KEY_GTKRC);
+
     GdmTimedLoginEnable = ve_config_get_bool (cfg, GDM_KEY_TIMED_LOGIN_ENABLE);
     GdmTimedLogin = ve_config_get_string (cfg, GDM_KEY_TIMED_LOGIN);
     GdmTimedLoginDelay = ve_config_get_int (cfg, GDM_KEY_TIMED_LOGIN_DELAY);
@@ -306,7 +364,6 @@ gdm_config_parse (void)
     GdmAllowRemoteAutoLogin = ve_config_get_bool (cfg, GDM_KEY_ALLOWREMOTEAUTOLOGIN);
     GdmRelaxPerms = ve_config_get_int (cfg, GDM_KEY_RELAXPERM);
     GdmUserMaxFile = ve_config_get_int (cfg, GDM_KEY_MAXFILE);
-    GdmSessionMaxFile = ve_config_get_int (cfg, GDM_KEY_SESSIONMAXFILE);
 
     GdmXdmcp = ve_config_get_bool (cfg, GDM_KEY_XDMCP);
     GdmDispPerHost = ve_config_get_int (cfg, GDM_KEY_DISPERHOST);
@@ -363,7 +420,7 @@ gdm_config_parse (void)
     }
 
     if (GdmAutomaticLogin != NULL &&
-	strcmp (GdmAutomaticLogin, "root") == 0) {
+	strcmp (GdmAutomaticLogin, gdm_root_user ()) == 0) {
 	    gdm_info (_("%s: Root cannot be autologged in, turing off automatic login"), "gdm_config_parse");
 	    g_free (GdmAutomaticLogin);
 	    GdmAutomaticLogin = NULL;
@@ -376,7 +433,7 @@ gdm_config_parse (void)
     }
 
     if (GdmTimedLogin != NULL &&
-	strcmp (GdmTimedLogin, "root") == 0) {
+	strcmp (GdmTimedLogin, gdm_root_user ()) == 0) {
 	    gdm_info (_("%s: Root cannot be autologged in, turing off timed login"), "gdm_config_parse");
 	    g_free (GdmTimedLogin);
 	    GdmTimedLogin = NULL;
@@ -399,16 +456,6 @@ gdm_config_parse (void)
 	    gdm_error (_("%s: No remote greeter specified."), "gdm_config_parse");
     }
 
-    if (ve_string_empty (GdmServAuthDir)) {
-	    gdm_text_message_dialog
-		    (_("No daemon/ServAuthDir specified in the configuration file"));
-	    GdmPidFile = NULL;
-	    gdm_fail (_("%s: No authdir specified."), "gdm_config_parse");
-    }
-
-    if (ve_string_empty (GdmLogDir))
-	GdmLogDir = GdmServAuthDir;
-
     if (ve_string_empty (GdmSessDir))
 	gdm_error (_("%s: No sessions directory specified."), "gdm_config_parse");
 
@@ -463,7 +510,9 @@ gdm_config_parse (void)
 
     /* Find local X server definitions */
     list = ve_config_get_keys (cfg, GDM_KEY_SERVERS);
-    for (li = list; li != NULL; li = li->next) {
+    /* only read the list if no_console is FALSE
+       at this stage */
+    for (li = list; ! no_console && li != NULL; li = li->next) {
 	    const char *key = li->data;
 	    if (isdigit (*key)) {
 		    char *full;
@@ -502,6 +551,13 @@ gdm_config_parse (void)
 
     if (displays == NULL && ! GdmXdmcp) {
 	    char *server = NULL;
+
+	    /* if we requested no local servers (there is no console),
+	       then don't display errors in console messages */
+	    if (no_console) {
+		    gdm_fail (_("%s: XDMCP disabled and no local servers defined. Aborting!"), "gdm_config_parse");
+	    }
+
 	    if (access (GdmStandardXServer, X_OK) == 0) {
 		    server = GdmStandardXServer;
 	    } else if (access ("/usr/bin/X11/X", X_OK) == 0) {
@@ -542,6 +598,11 @@ gdm_config_parse (void)
 	    }
     }
 
+    /* If no displays were found, then obviously
+       we're in a no console mode */
+    if (displays == NULL)
+	    no_console = TRUE;
+
     /* Lookup user and groupid for the gdm user */
     pwent = getpwnam (GdmUser);
 
@@ -558,7 +619,8 @@ gdm_config_parse (void)
 		       "Please correct gdm configuration %s "
 		       "and restart gdm."),
 		     GDM_CONFIG_FILE);
-	    gdm_text_message_dialog (s);
+	    if ( ! no_console)
+		    gdm_text_message_dialog (s);
 	    GdmPidFile = NULL;
 	    gdm_fail (_("%s: Can't find the gdm user (%s). Aborting!"), "gdm_config_parse", GdmUser);
     } else {
@@ -572,7 +634,8 @@ gdm_config_parse (void)
 		       "pose a security risk.  Please "
 		       "correct gdm configuration %s and "
 		       "restart gdm."), GDM_CONFIG_FILE);
-	    gdm_text_message_dialog (s);
+	    if ( ! no_console)
+		    gdm_text_message_dialog (s);
 	    GdmPidFile = NULL;
 	    gdm_fail (_("%s: The gdm user should not be root. Aborting!"), "gdm_config_parse");
     }
@@ -583,7 +646,7 @@ gdm_config_parse (void)
 	    gdm_error (_("%s: Can't find the gdm group (%s). Trying 'nobody'!"), "gdm_config_parse", GdmUser);
 	    g_free (GdmGroup);
 	    GdmGroup = g_strdup ("nobody");
-	    pwent = getpwnam (GdmUser);
+	    grent = getgrnam (GdmGroup);
     }
 
     if (grent == NULL) {
@@ -592,7 +655,8 @@ gdm_config_parse (void)
 		       "Please correct gdm configuration %s "
 		       "and restart gdm."),
 		     GDM_CONFIG_FILE);
-	    gdm_text_message_dialog (s);
+	    if ( ! no_console)
+		    gdm_text_message_dialog (s);
 	    GdmPidFile = NULL;
 	    gdm_fail (_("%s: Can't find the gdm group (%s). Aborting!"), "gdm_config_parse", GdmGroup);
     } else  {
@@ -606,7 +670,8 @@ gdm_config_parse (void)
 		       "pose a security risk. Please "
 		       "correct gdm configuration %s and "
 		       "restart gdm."), GDM_CONFIG_FILE);
-	    gdm_text_message_dialog (s);
+	    if ( ! no_console)
+		    gdm_text_message_dialog (s);
 	    GdmPidFile = NULL;
 	    gdm_fail (_("%s: The gdm group should not be root. Aborting!"), "gdm_config_parse");
     }
@@ -646,35 +711,37 @@ gdm_config_parse (void)
     
     g_free (bin);
 
+    /* Check the serv auth and log dirs */
+    if (ve_string_empty (GdmServAuthDir)) {
+	    if ( ! no_console)
+		    gdm_text_message_dialog
+			    (_("No daemon/ServAuthDir specified in the configuration file"));
+	    GdmPidFile = NULL;
+	    gdm_fail (_("%s: No daemon/ServAuthDir specified."), "gdm_config_parse");
+    }
+
+    if (ve_string_empty (GdmLogDir)) {
+	g_free (GdmLogDir);
+	GdmLogDir = g_strdup (GdmServAuthDir);
+    }
 
     /* Enter paranoia mode */
-    if (stat (GdmServAuthDir, &statbuf) == -1)  {
-	    char *s = g_strdup_printf
-		    (_("Server Authorization directory "
-		       "(daemon/ServAuthDir) is set to %s "
-		       "but this does not exist. Please "
-		       "correct gdm configuration %s and "
-		       "restart gdm."), GdmServAuthDir,
-		     GDM_CONFIG_FILE);
-	    gdm_text_message_dialog (s);
-	    GdmPidFile = NULL;
-	    gdm_fail (_("%s: Authdir %s does not exist. Aborting."), "gdm_config_parse", GdmServAuthDir);
-    }
+    check_servauthdir (&statbuf);
 
-    if (! S_ISDIR (statbuf.st_mode)) {
-	    char *s = g_strdup_printf
-		    (_("Server Authorization directory "
-		       "(daemon/ServAuthDir) is set to %s "
-		       "but this is not a directory. Please "
-		       "correct gdm configuration %s and "
-		       "restart gdm."), GdmServAuthDir,
-		     GDM_CONFIG_FILE);
-	    gdm_text_message_dialog (s);
-	    GdmPidFile = NULL;
-	    gdm_fail (_("%s: Authdir %s is not a directory. Aborting."), "gdm_config_parse", GdmServAuthDir);
-    }
+    seteuid (0);
+    setegid (0);
 
-    if (statbuf.st_uid != GdmUserId || statbuf.st_gid != GdmGroupId)  {
+    /* Now set things up for us as  */
+    chown (GdmServAuthDir, 0, GdmGroupId);
+    chmod (GdmServAuthDir, (S_IRWXU|S_IRWXG|S_ISVTX));
+
+    setegid (GdmGroupId);
+    seteuid (GdmUserId);
+
+    /* again paranoid */
+    check_servauthdir (&statbuf);
+
+    if (statbuf.st_uid != 0 || statbuf.st_gid != GdmGroupId)  {
 	    char *s = g_strdup_printf
 		    (_("Server Authorization directory "
 		       "(daemon/ServAuthDir) is set to %s "
@@ -684,31 +751,35 @@ gdm_config_parse (void)
 		       "gdm."),
 		     GdmServAuthDir, GdmUser, GdmGroup,
 		     GDM_CONFIG_FILE);
-	    gdm_text_message_dialog (s);
+	    if ( ! no_console)
+		    gdm_text_message_dialog (s);
 	    GdmPidFile = NULL;
 	    gdm_fail (_("%s: Authdir %s is not owned by user %s, group %s. Aborting."), "gdm_config_parse", 
-		      GdmServAuthDir, GdmUser, GdmGroup);
+		      GdmServAuthDir, gdm_root_user (), GdmGroup);
     }
 
-    if (statbuf.st_mode != (S_IFDIR|S_IRWXU|S_IRGRP|S_IXGRP))  {
+    if (statbuf.st_mode != (S_IFDIR|S_IRWXU|S_IRWXG|S_ISVTX))  {
 	    char *s = g_strdup_printf
 		    (_("Server Authorization directory "
 		       "(daemon/ServAuthDir) is set to %s "
 		       "but has the wrong permissions, it "
-		       "should have permissions of 0750. "
+		       "should have permissions of %o. "
 		       "Please correct the permissions or "
 		       "the gdm configuration %s and "
 		       "restart gdm."),
-		     GdmServAuthDir, GDM_CONFIG_FILE);
-	    gdm_text_message_dialog (s);
+		     GdmServAuthDir, (S_IRWXU|S_IRWXG|S_ISVTX), GDM_CONFIG_FILE);
+	    if ( ! no_console)
+		    gdm_text_message_dialog (s);
 	    GdmPidFile = NULL;
-	    gdm_fail (_("%s: Authdir %s has wrong permissions %o. Should be 0750. Aborting."), "gdm_config_parse", 
-		      GdmServAuthDir, statbuf.st_mode);
+	    gdm_fail (_("%s: Authdir %s has wrong permissions %o. Should be %o. Aborting."), "gdm_config_parse", 
+		      GdmServAuthDir, statbuf.st_mode, (S_IRWXU|S_IRWXG|S_ISVTX));
     }
 
     seteuid (0);
     setegid (0);
 
+    check_logdir ();
+
     /* Check that user authentication is properly configured */
     gdm_verify_check ();
 
@@ -757,6 +828,7 @@ gdm_daemonify (void)
 
         exit (EXIT_SUCCESS);
     }
+    gdm_main_pid = getpid ();
 
     if (pid < 0) 
 	gdm_fail (_("%s: fork() failed!"), "gdm_daemonify");
@@ -765,12 +837,12 @@ gdm_daemonify (void)
 	gdm_fail (_("%s: setsid() failed: %s!"), "gdm_daemonify",
 		  strerror(errno));
 
-    chdir (GdmServAuthDir);
+    IGNORE_EINTR (chdir (GdmServAuthDir));
     umask (022);
 
-    close (0);
-    close (1);
-    close (2);
+    IGNORE_EINTR (close (0));
+    IGNORE_EINTR (close (1));
+    IGNORE_EINTR (close (2));
 
     gdm_open_dev_null (O_RDONLY); /* open stdin - fd 0 */
     gdm_open_dev_null (O_RDWR); /* open stdout - fd 1 */
@@ -817,10 +889,11 @@ void
 gdm_final_cleanup (void)
 {
 	GSList *list, *li;
+	gboolean first;
 
 	gdm_debug ("gdm_final_cleanup");
 
-	gdm_sigchld_block_push ();
+	gdm_in_final_cleanup = TRUE;
 
 	if (extra_process > 1) {
 		/* we sigterm extra processes, and we
@@ -829,7 +902,47 @@ gdm_final_cleanup (void)
 		extra_process = 0;
 	}
 
-	gdm_sigchld_block_pop ();
+	/* First off whack all XDMCP and FLEXI_XNEST
+	   slaves, we'll wait for them later */
+	for (li = displays; li != NULL; li = li->next) {
+		GdmDisplay *d = li->data;
+		if (d->type == TYPE_XDMCP ||
+		    d->type == TYPE_FLEXI_XNEST) {
+			/* set to DEAD so that we won't kill it again */
+			d->dispstat = DISPLAY_DEAD;
+			if (d->slavepid > 1)
+				kill (d->slavepid, SIGTERM);
+		}
+	}
+
+	/* Now completely unmanage the local servers */
+	first = TRUE;
+	list = g_slist_copy (displays);
+	/* somewhat of a hack to kill last server
+	 * started first.  This mostly makes things end up on
+	 * the right vt */
+	list = g_slist_reverse (list);
+	for (li = list; li != NULL; li = li->next) {
+		GdmDisplay *d = li->data;
+		if (d->type == TYPE_XDMCP ||
+		    d->type == TYPE_FLEXI_XNEST)
+			continue;
+		/* HACK! Wait 2 seconds between killing of local servers
+		 * because X is stupid and full of races and will hang my
+		 * keyboard if I don't */
+		if ( ! first) {
+			/* there could be signals happening
+			   here */
+			gdm_sleep_no_signal (2);
+		}
+		first = FALSE;
+		gdm_display_unmanage (d);
+	}
+	g_slist_free (list);
+
+	/* and now kill and wait for the XDMCP and FLEXI_XNEST
+	   slaves.  unmanage will not kill slaves we have already
+	   killed unless a SIGTERM was sent in the meantime */
 
 	list = g_slist_copy (displays);
 	for (li = list; li != NULL; li = li->next) {
@@ -842,21 +955,6 @@ gdm_final_cleanup (void)
 	}
 	g_slist_free (list);
 
-	list = g_slist_copy (displays);
-	/* somewhat of a hack to kill last server
-	 * started first.  This mostly makes things end up on
-	 * the right vt */
-	list = g_slist_reverse (list);
-	for (li = list; li != NULL; li = li->next) {
-		GdmDisplay *d = li->data;
-		/* HACK! Wait 2 seconds between killing of local servers
-		 * because X is stupid and full of races and will hang my
-		 * keyboard if I don't */
-		if (li != list)
-			sleep (2);
-		gdm_display_unmanage (d);
-	}
-	g_slist_free (list);
 
 	/* Close stuff */
 
@@ -867,21 +965,31 @@ gdm_final_cleanup (void)
 		char *path;
 		gdm_connection_close (fifoconn);
 		path = g_strconcat (GdmServAuthDir, "/.gdmfifo", NULL);
-		unlink (path);
+		IGNORE_EINTR (unlink (path));
 		g_free (path);
 		fifoconn = NULL;
 	}
 
+	if (pipeconn != NULL) {
+		gdm_connection_close (pipeconn);
+		pipeconn = NULL;
+	}
+
+	if (slave_fifo_pipe_fd >= 0) {
+		IGNORE_EINTR (close (slave_fifo_pipe_fd));
+		slave_fifo_pipe_fd = -1;
+	}
+
 	if (unixconn != NULL) {
 		gdm_connection_close (unixconn);
-		unlink (GDM_SUP_SOCKET);
+		IGNORE_EINTR (unlink (GDM_SUP_SOCKET));
 		unixconn = NULL;
 	}
 
 	closelog();
 
 	if (GdmPidFile != NULL)
-		unlink (GdmPidFile);
+		IGNORE_EINTR (unlink (GdmPidFile));
 }
 
 static gboolean
@@ -917,15 +1025,27 @@ deal_with_x_crashes (GdmDisplay *d)
 			"XKeepsCrashing script"),
 		      "deal_with_x_crashes");
 
-	    pid = gdm_fork_extra ();
+	    extra_process = pid = fork ();
+	    if (pid < 0)
+		    extra_process = 0;
 
 	    if (pid == 0) {
 		    char *argv[2];
 		    char *xlog = g_strconcat (GdmLogDir, "/", d->name, ".log", NULL);
 
+		    gdm_unset_signals ();
+
+		    /* Also make a new process group so that we may use
+		     * kill -(extra_process) to kill extra process and all it's
+		     * possible children */
+		    setsid ();
+
+		    if (GdmXdmcp)
+			    gdm_xdmcp_close ();
+
 		    closelog ();
 
-		    gdm_close_all_descriptors (0 /* from */, -1 /* except */);
+		    gdm_close_all_descriptors (0 /* from */, -1 /* except */, -1 /* except2 */);
 
 		    /* No error checking here - if it's messed the best response
 		    * is to ignore & try to continue */
@@ -954,14 +1074,36 @@ deal_with_x_crashes (GdmDisplay *d)
 		    ve_setenv ("TEXTDOMAIN", GETTEXT_PACKAGE, TRUE);
 		    ve_setenv ("TEXTDOMAINDIR", GNOMELOCALEDIR, TRUE);
 
-		    execv (argv[0], argv);
+		    IGNORE_EINTR (execv (argv[0], argv));
 	
 		    /* yaikes! */
 		    _exit (32);
 	    } else if (pid > 0) {
 		    int status;
 
-		    gdm_wait_for_extra (&status);
+		    if (extra_process > 1) {
+			    int ret;
+			    int killsignal = SIGTERM;
+			    int storeerrno;
+			    errno = 0;
+			    ret = waitpid (extra_process, &status, WNOHANG);
+			    do {
+				    /* wait for some signal, yes this is a race */
+				    if (ret <= 0)
+					    sleep (10);
+				    errno = 0;
+				    ret = waitpid (extra_process, &status, WNOHANG);
+				    storeerrno = errno;
+				    if (ret <= 0 &&
+					(ve_signal_was_notified (SIGTERM) ||
+					 ve_signal_was_notified (SIGINT) ||
+					 ve_signal_was_notified (SIGHUP))) {
+					    kill (-(extra_process), killsignal);
+					    killsignal = SIGKILL;
+				    }
+			    } while (ret == 0 || (ret < 0 && storeerrno == EINTR));
+		    }
+		    extra_process = 0;
 
 		    if (WIFEXITED (status) &&
 			WEXITSTATUS (status) == 0) {
@@ -1023,10 +1165,10 @@ suspend_machine (void)
 		/* Also make a new process group */
 		setsid ();
 
-		chdir ("/");
+		IGNORE_EINTR (chdir ("/"));
 
 		argv = ve_split (GdmSuspendReal);
-		execv (argv[0], argv);
+		IGNORE_EINTR (execv (argv[0], argv));
 		/* FIXME: what about fail */
 		_exit (1);
 	}
@@ -1037,8 +1179,8 @@ static void
 change_to_first_and_clear (gboolean reboot)
 {
 	gdm_change_vt (1);
-	close (1);
-	close (2);
+	IGNORE_EINTR (close (1));
+	IGNORE_EINTR (close (2));
 	open ("/dev/tty1", O_WRONLY);
 	open ("/dev/tty1", O_WRONLY);
 	/* yet again lazy */
@@ -1074,14 +1216,25 @@ gdm_cleanup_children (void)
     if (WIFEXITED (exitstatus)) {
 	    status = WEXITSTATUS (exitstatus);
 	    crashed = FALSE;
+	    gdm_debug ("gdm_cleanup_children: child %d returned %d", pid, status);
     } else {
 	    status = EXIT_SUCCESS;
 	    crashed = TRUE;
+	    if (WIFSIGNALED (exitstatus)) {
+		    if (WTERMSIG (exitstatus) == SIGTERM ||
+			WTERMSIG (exitstatus) == SIGINT) {
+			    /* we send these signals, sometimes children don't handle them */
+			    gdm_debug ("gdm_cleanup_children: child %d died of signal %d (TERM/INT)", pid,
+				       (int)WTERMSIG (exitstatus));
+		    } else {
+			    gdm_error ("gdm_cleanup_children: child %d crashed of signal %d", pid,
+				       (int)WTERMSIG (exitstatus));
+		    }
+	    } else {
+		    gdm_error ("gdm_cleanup_children: child %d crashed", pid);
+	    }
     }
 	
-    gdm_debug ("gdm_cleanup_children: child %d returned %d%s", pid, status,
-	       crashed ? " (child crashed)" : "");
-
     if (pid == extra_process) {
 	    /* an extra process died, yay! */
 	    extra_process = 0;
@@ -1096,25 +1249,24 @@ gdm_cleanup_children (void)
 	    return TRUE;
 
     if (crashed) {
-	    gdm_debug ("gdm_cleanup_children: Slave crashed, killing it's "
+	    gdm_error ("gdm_cleanup_children: Slave crashed, killing its "
 		       "children");
 
-	    if (d->sesspid > 1 &&
-		kill (d->sesspid, SIGTERM) == 0)
-			    ve_waitpid_no_signal (d->sesspid, NULL, 0);
+	    if (d->sesspid > 1)
+		    kill (-(d->sesspid), SIGTERM);
 	    d->sesspid = 0;
-	    if (d->greetpid > 1 &&
-		kill (d->greetpid, SIGTERM) == 0)
-			    ve_waitpid_no_signal (d->greetpid, NULL, 0);
+	    if (d->greetpid > 1)
+		    kill (-(d->greetpid), SIGTERM);
 	    d->greetpid = 0;
-	    if (d->chooserpid > 1 &&
-		kill (d->chooserpid, SIGTERM) == 0)
-			    ve_waitpid_no_signal (d->chooserpid, NULL, 0);
+	    if (d->chooserpid > 1)
+		    kill (-(d->chooserpid), SIGTERM);
 	    d->chooserpid = 0;
-	    if (d->servpid > 1 &&
-		kill (d->servpid, SIGTERM) == 0)
-			    ve_waitpid_no_signal (d->servpid, NULL, 0);
+	    if (d->servpid > 1)
+		    kill (d->servpid, SIGTERM);
 	    d->servpid = 0;
+
+	    /* race avoider */
+	    gdm_sleep_no_signal (1);
     }
 
     /* null all these, they are not valid most definately */
@@ -1219,14 +1371,14 @@ start_autopsy:
 	gdm_info (_("Master rebooting..."));
 
 	gdm_final_cleanup ();
-	chdir ("/");
+	IGNORE_EINTR (chdir ("/"));
 
 #ifdef __linux__
 	change_to_first_and_clear (TRUE /* reboot */);
 #endif /* __linux */
 
 	argv = ve_split (GdmRebootReal);
-	execv (argv[0], argv);
+	IGNORE_EINTR (execv (argv[0], argv));
 
 	gdm_error (_("%s: Reboot failed: %s"), 
 		   "gdm_child_action", strerror (errno));
@@ -1239,14 +1391,14 @@ start_autopsy:
 	gdm_info (_("Master halting..."));
 
 	gdm_final_cleanup ();
-	chdir ("/");
+	IGNORE_EINTR (chdir ("/"));
 
 #ifdef __linux__
 	change_to_first_and_clear (FALSE /* reboot */);
 #endif /* __linux */
 
 	argv = ve_split (GdmHaltReal);
-	execv (argv[0], argv);
+	IGNORE_EINTR (execv (argv[0], argv));
 
 	gdm_error (_("%s: Halt failed: %s"),
 		   "gdm_child_action", strerror (errno));
@@ -1270,6 +1422,7 @@ start_autopsy:
 	break;
 
     case DISPLAY_XFAILED:       /* X sucks */
+	gdm_debug ("X failed!");
 	/* inform about error if needed */
 	if (d->socket_conn != NULL) {
 		GdmConnection *conn = d->socket_conn;
@@ -1317,7 +1470,7 @@ start_autopsy:
 
     case DISPLAY_REMANAGE:	/* Remanage display */
     default:
-	gdm_debug ("gdm_child_action: Slave process returned %d", status);
+	gdm_debug ("gdm_child_action: In remanage");
 
 	/* inform about error if needed */
 	if (d->socket_conn != NULL) {
@@ -1370,7 +1523,7 @@ gdm_restart_now (void)
 	gdm_info (_("GDM restarting ..."));
 	gdm_final_cleanup ();
 	gdm_restoreenv ();
-	execvp (stored_argv[0], stored_argv);
+	IGNORE_EINTR (execvp (stored_argv[0], stored_argv));
 	gdm_error (_("Failed to restart self"));
 	_exit (1);
 }
@@ -1390,6 +1543,15 @@ gdm_safe_restart (void)
 	gdm_restart_now ();
 }
 
+static void
+main_daemon_abrt (int sig)
+{
+	/* FIXME: note that this could mean out of memory */
+	gdm_error ("main daemon: Got SIGABRT, something went very wrong. Going down!");
+	gdm_final_cleanup ();
+	exit (EXIT_FAILURE);
+}
+
 static gboolean
 mainloop_sig_callback (int sig, gpointer data)
 {
@@ -1408,6 +1570,20 @@ mainloop_sig_callback (int sig, gpointer data)
       exit (EXIT_SUCCESS);
       break;
 
+#ifdef SIGXFSZ
+    case SIGXFSZ:
+      gdm_error ("main daemon: Hit file size rlimit, restarting!");
+      gdm_restart_now ();
+      break;
+#endif
+
+#ifdef SIGXCPU
+    case SIGXCPU:
+      gdm_error ("main daemon: Hit CPU rlimit, restarting!");
+      gdm_restart_now ();
+      break;
+#endif
+
     case SIGHUP:
       gdm_restart_now ();
       break;
@@ -1440,33 +1616,74 @@ store_argv (int argc, char *argv[])
 	stored_argc = argc;
 }
 
+static void
+close_notify (gpointer data)
+{
+	GdmConnection **conn = data;
+	* conn = NULL;
+}
+
 static void
 create_connections (void)
 {
+	int p[2];
 	gchar *path;
 
 	path = g_strconcat (GdmServAuthDir, "/.gdmfifo", NULL);
 	fifoconn = gdm_connection_open_fifo (path, 0660);
 	g_free (path);
 
-	if (fifoconn != NULL)
+	if (fifoconn != NULL) {
 		gdm_connection_set_handler (fifoconn,
 					    gdm_handle_message,
 					    NULL /* data */,
 					    NULL /* destroy_notify */);
+		gdm_connection_set_close_notify (fifoconn,
+						 &fifoconn,
+						 close_notify);
+	}
+
+	if (pipe (p) < 0) {
+		slave_fifo_pipe_fd = -1;
+		pipeconn = NULL;
+	} else {
+		slave_fifo_pipe_fd = p[1];
+		pipeconn = gdm_connection_open_fd (p[0]);
+	}
+
+	if (pipeconn != NULL) {
+		gdm_connection_set_handler (pipeconn,
+					    gdm_handle_message,
+					    NULL /* data */,
+					    NULL /* destroy_notify */);
+		gdm_connection_set_close_notify (pipeconn,
+						 &pipeconn,
+						 close_notify);
+	} else {
+		IGNORE_EINTR (close (p[0]));
+		IGNORE_EINTR (close (p[1]));
+		slave_fifo_pipe_fd = -1;
+	}
+
 
 	unixconn = gdm_connection_open_unix (GDM_SUP_SOCKET, 0666);
 
-	if (unixconn != NULL)
+	if (unixconn != NULL) {
 		gdm_connection_set_handler (unixconn,
 					    gdm_handle_user_message,
 					    NULL /* data */,
 					    NULL /* destroy_notify */);
+		gdm_connection_set_close_notify (unixconn,
+						 &unixconn,
+						 close_notify);
+	}
 }
 
 struct poptOption options [] = {
 	{ "nodaemon", '\0', POPT_ARG_NONE | POPT_ARGFLAG_ONEDASH,
 	  &no_daemon, 0, N_("Do not fork into the background"), NULL },
+	{ "no-console", '\0', POPT_ARG_NONE,
+	  &no_console, 0, N_("No console (local) servers to be run"), NULL },
 	{ "preserve-ld-vars", '\0', POPT_ARG_NONE,
 	  &preserve_ld_vars, 0, N_("Preserve LD_* variables"), NULL },
         POPT_AUTOHELP
@@ -1512,7 +1729,7 @@ ensure_desc_012 (void)
 		/* Once we are up to 3, we're beyond stdin,
 		 * stdout and stderr */
 		if (fd >= 3) {
-			close (fd);
+			IGNORE_EINTR (close (fd));
 			break;
 		}
 	}
@@ -1536,11 +1753,21 @@ gdm_get_our_runlevel (void)
 #endif /* __linux__ */
 }
 
+/* initially if we get a TERM or INT we just want to die,
+   but we want to also kill an extra process if it exists */
+static void
+initial_term_int (int signal)
+{
+	if (extra_process > 1)
+		kill (-(extra_process), SIGTERM);
+	_exit (EXIT_FAILURE);
+}
+
 int 
 main (int argc, char *argv[])
 {
     sigset_t mask;
-    struct sigaction term, child;
+    struct sigaction sig, child, abrt;
     FILE *pf;
     poptContext ctx;
     int nextopt;
@@ -1551,9 +1778,10 @@ main (int argc, char *argv[])
     /* We here ensure descriptors 0, 1 and 2 */
     ensure_desc_012 ();
 
+    /* store all initial stuff, args, env, rlimits, runlevel */
     store_argv (argc, argv);
     gdm_saveenv ();
-
+    gdm_get_initial_limits ();
     gdm_get_our_runlevel ();
 
     bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -1570,10 +1798,11 @@ main (int argc, char *argv[])
 	/* do nothing */ ;
 
     if (nextopt != -1) {
-	    printf (_("Error on option %s: %s.\nRun '%s --help' to see a full list of available command line options.\n"),
-		    poptBadOption (ctx, 0),
-		    poptStrerror (nextopt),
-		    argv[0]);
+	    fprintf (stderr,
+		     _("Error on option %s: %s.\nRun '%s --help' to see a full list of available command line options.\n"),
+		     poptBadOption (ctx, 0),
+		     poptStrerror (nextopt),
+		     argv[0]);
 	    fflush (stdout);
 	    exit (1);
     }
@@ -1592,6 +1821,22 @@ main (int argc, char *argv[])
 	    gdm_charset = g_strdup (charset);
     }
 
+    /* initial TERM/INT handler */
+    sig.sa_handler = initial_term_int;
+    sig.sa_flags = SA_RESTART;
+    sigemptyset (&sig.sa_mask);
+
+    if (sigaction (SIGTERM, &sig, NULL) < 0) 
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
+		  "main", "TERM", strerror (errno));
+
+    if (sigaction (SIGINT, &sig, NULL) < 0) 
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
+		  "main", "INT", strerror (errno));
+
+    /* get the name of the root user */
+    gdm_root_user ();
+
     /* Parse configuration file */
     gdm_config_parse();
 
@@ -1626,7 +1871,7 @@ main (int argc, char *argv[])
 	    fclose (pf);
 	}
 
-	chdir (GdmServAuthDir);
+	IGNORE_EINTR (chdir (GdmServAuthDir));
 	umask (022);
     }
     else
@@ -1638,26 +1883,53 @@ main (int argc, char *argv[])
     ve_signal_add (SIGINT, mainloop_sig_callback, NULL);
     ve_signal_add (SIGHUP, mainloop_sig_callback, NULL);
     ve_signal_add (SIGUSR1, mainloop_sig_callback, NULL);
-    
-    term.sa_handler = ve_signal_notify;
-    term.sa_flags = SA_RESTART;
-    sigemptyset (&term.sa_mask);
 
-    if (sigaction (SIGTERM, &term, NULL) < 0) 
-	gdm_fail (_("%s: Error setting up TERM signal handler"),
-		  "gdm_main");
+    sig.sa_handler = ve_signal_notify;
+    sig.sa_flags = SA_RESTART;
+    sigemptyset (&sig.sa_mask);
 
-    if (sigaction (SIGINT, &term, NULL) < 0) 
-	gdm_fail (_("%s: Error setting up INT signal handler"),
-		  "gdm_main");
+    if (sigaction (SIGTERM, &sig, NULL) < 0) 
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
+		  "main", "TERM", strerror (errno));
 
-    if (sigaction (SIGHUP, &term, NULL) < 0) 
-	gdm_fail (_("%s: Error setting up HUP signal handler"),
-		  "gdm_main");
+    if (sigaction (SIGINT, &sig, NULL) < 0) 
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
+		  "main", "INT", strerror (errno));
 
-    if (sigaction (SIGUSR1, &term, NULL) < 0) 
-	gdm_fail (_("%s: Error setting up USR1 signal handler"),
-		  "gdm_main");
+    if (sigaction (SIGHUP, &sig, NULL) < 0) 
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
+		  "main", "HUP", strerror (errno));
+
+    if (sigaction (SIGUSR1, &sig, NULL) < 0) 
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
+		  "main", "USR1", strerror (errno));
+
+    /* some process limit signals we catch and restart on,
+       note that we don't catch these in the slave, but then
+       we catch those in the main daemon as slave crashing
+       (terminated by signal), and we clean up appropriately */
+#ifdef SIGXCPU
+    ve_signal_add (SIGXCPU, mainloop_sig_callback, NULL);
+    if (sigaction (SIGXCPU, &sig, NULL) < 0) 
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
+		  "main", "XCPU", strerror (errno));
+#endif
+#ifdef SIGXFSZ
+    ve_signal_add (SIGXFSZ, mainloop_sig_callback, NULL);
+    if (sigaction (SIGXFSZ, &sig, NULL) < 0) 
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
+		  "main", "XFSZ", strerror (errno));
+#endif
+
+    /* cannot use mainloop for SIGABRT, the handler can never
+       return */
+    abrt.sa_handler = main_daemon_abrt;
+    abrt.sa_flags = SA_RESTART;
+    sigemptyset (&abrt.sa_mask);
+
+    if (sigaction (SIGABRT, &abrt, NULL) < 0) 
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
+		  "main", "ABRT", strerror (errno));
 
     child.sa_handler = ve_signal_notify;
     child.sa_flags = SA_RESTART|SA_NOCLDSTOP;
@@ -1673,8 +1945,27 @@ main (int argc, char *argv[])
     sigaddset (&mask, SIGCHLD);
     sigaddset (&mask, SIGHUP);
     sigaddset (&mask, SIGUSR1);
+    sigaddset (&mask, SIGABRT);
+#ifdef SIGXCPU
+    sigaddset (&mask, SIGXCPU);
+#endif
+#ifdef SIGXFSZ
+    sigaddset (&mask, SIGXFSZ);
+#endif
     sigprocmask (SIG_UNBLOCK, &mask, NULL);
 
+    gdm_signal_ignore (SIGUSR2);
+
+    /* ignore power failures, up to user processes to
+     * handle things correctly */
+#ifdef SIGPWR
+    gdm_signal_ignore (SIGPWR);
+#endif
+    /* can we ever even get this one? */
+#ifdef SIGLOST
+    gdm_signal_ignore (SIGLOST);
+#endif
+
     gdm_debug ("gdm_main: Here we go...");
 
     /* Init XDMCP if applicable */
@@ -1683,6 +1974,9 @@ main (int argc, char *argv[])
 
     create_connections ();
 
+    /* make sure things (currently /tmp/.ICE-unix) are sane */
+    gdm_ensure_sanity () ;
+
     /* Start local X servers */
     gdm_start_first_unborn_local (0 /* delay */);
 
@@ -1785,20 +2079,23 @@ send_slave_ack (GdmDisplay *d, const char *resp)
 			char not[2];
 			not[0] = GDM_SLAVE_NOTIFY_ACK;
 			not[1] = '\n';
-			write (d->master_notify_fd, not, 2);
+			IGNORE_EINTR (write (d->master_notify_fd, not, 2));
 		} else {
 			char *not = g_strdup_printf ("%c%s\n",
 						     GDM_SLAVE_NOTIFY_ACK,
 						     resp);
-			write (d->master_notify_fd, not, strlen (not));
+			IGNORE_EINTR (write (d->master_notify_fd, not, strlen (not)));
 			g_free (not);
 		}
 	}
-	gdm_sigchld_block_push ();
 	if (d->slavepid > 1) {
 		kill (d->slavepid, SIGUSR2);
+		/* now yield the CPU as the other process has more
+		   useful work to do then we do */
+#if defined(_POSIX_PRIORITY_SCHEDULING) && defined(HAVE_SCHED_YIELD)
+		sched_yield ();
+#endif
 	}
-	gdm_sigchld_block_pop ();
 }
 
 static void
@@ -1808,14 +2105,17 @@ send_slave_command (GdmDisplay *d, const char *command)
 		char *cmd = g_strdup_printf ("%c%s\n",
 					     GDM_SLAVE_NOTIFY_COMMAND,
 					     command);
-		write (d->master_notify_fd, cmd, strlen (cmd));
+		IGNORE_EINTR (write (d->master_notify_fd, cmd, strlen (cmd)));
 		g_free (cmd);
 	}
-	gdm_sigchld_block_push ();
 	if (d->slavepid > 1) {
 		kill (d->slavepid, SIGUSR2);
+		/* now yield the CPU as the other process has more
+		   useful work to do then we do */
+#if defined(_POSIX_PRIORITY_SCHEDULING) && defined(HAVE_SCHED_YIELD)
+		sched_yield ();
+#endif
 	}
-	gdm_sigchld_block_pop ();
 }
 
 
@@ -2220,10 +2520,10 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 		GSList *li;
 		for (li = displays; li != NULL; li = li->next) {
 			GdmDisplay *d = li->data;
-			gdm_sigchld_block_push ();
 			if (d->greetpid > 1)
 				kill (d->greetpid, SIGHUP);
-			gdm_sigchld_block_pop ();
+			else if (d->chooserpid > 1)
+				kill (d->chooserpid, SIGHUP);
 		}
 	} else if (strncmp (msg, GDM_SOP_WRITE_X_SERVERS " ",
 		            strlen (GDM_SOP_WRITE_X_SERVERS " ")) == 0) {
@@ -2392,7 +2692,7 @@ dehex_cookie (const char *cookie, int *len)
 	for (i = 0, p = cookie;
 	     *p != '\0' && *(p+1) != '\0';
 	     i++, p += 2) {
-		int num;
+		unsigned int num;
 		if (sscanf (p, "%02x", &num) != 1) {
 			g_free (bcookie);
 			return NULL;
@@ -2464,6 +2764,7 @@ handle_flexi_server (GdmConnection *conn, int type, const char *server,
 
 	if (type == TYPE_FLEXI_XNEST) {
 		struct stat s;
+		int r;
 		gboolean authorized = TRUE;
 
 		seteuid (xnest_uid);
@@ -2472,7 +2773,8 @@ handle_flexi_server (GdmConnection *conn, int type, const char *server,
 		g_assert (xnest_disp != NULL);
 		g_assert (xnest_cookie != NULL);
 
-		if (stat (xnest_auth_file, &s) < 0)
+		IGNORE_EINTR (r = stat (xnest_auth_file, &s));
+		if (r < 0)
 			authorized = FALSE;
 		if (authorized &&
 		    /* if readable or writable by group or others,
@@ -2614,10 +2916,8 @@ notify_displays_int (const char *key, int val)
 			gdm_fdprintf (disp->master_notify_fd,
 				      "%c%s %d\n",
 				      GDM_SLAVE_NOTIFY_KEY, key, val);
-			gdm_sigchld_block_push ();
 			if (disp->slavepid > 1)
 				kill (disp->slavepid, SIGUSR2);
-			gdm_sigchld_block_pop ();
 		}
 	}
 }
@@ -2632,10 +2932,8 @@ notify_displays_string (const char *key, const char *val)
 			gdm_fdprintf (disp->master_notify_fd,
 				      "%c%s %s\n",
 				      GDM_SLAVE_NOTIFY_KEY, key, val);
-			gdm_sigchld_block_push ();
 			if (disp->slavepid > 1)
 				kill (disp->slavepid, SIGUSR2);
-			gdm_sigchld_block_pop ();
 		}
 	}
 }
@@ -2645,8 +2943,10 @@ update_config (const char *key)
 {
 	struct stat statbuf;
 	VeConfig *cfg;
+	int r;
 
-	if (stat (GDM_CONFIG_FILE, &statbuf) == -1) {
+	IGNORE_EINTR (r = stat (GDM_CONFIG_FILE, &statbuf));
+	if (r < 0) {
 		/* if the file didn't exist before either */
 		if (config_file_mtime == 0)
 			return TRUE;
