@@ -64,6 +64,7 @@ get_free_display (void)
 	for (i = 20; i < 3000; i ++) {
 		struct stat s;
 		char buf[256];
+		FILE *fp;
 		sock = socket (AF_INET, SOCK_STREAM, 0);
 
 		serv_addr.sin_port = htons (6000 + i);
@@ -78,6 +79,23 @@ get_free_display (void)
 
 		close (sock);
 
+		/* if lock file exists and the process exists */
+		g_snprintf (buf, sizeof (buf), "/tmp/.X%d-lock", i);
+		fp = fopen (buf, "r");
+		if (fp != NULL) {
+			char buf2[100];
+			if (fgets (buf2, sizeof (buf2), fp) != NULL) {
+				gulong pid;
+				if (sscanf (buf2, "%lu", &pid) == 1 &&
+				    kill (pid, 0) == 0) {
+					fclose (fp);
+					continue;
+				}
+
+			}
+			fclose (fp);
+		}
+
 		g_snprintf (buf, sizeof (buf), "/tmp/.X11-unix/X%d", i);
 		if (stat (buf, &s) == 0 &&
 		    s.st_uid != getuid ()) {
@@ -461,14 +479,15 @@ main (int argc, char *argv[])
 
 	if (execvec == NULL) {
 		GtkWidget *d;
-		d = gtk_message_dialog_new (NULL /* parent */,
-					    GTK_DIALOG_MODAL /* flags */,
-					    GTK_MESSAGE_WARNING,
-					    GTK_BUTTONS_OK,
-					    _("Xnest doesn't exist.\n"
-					    "Please ask your system "
-					    "administrator\n"
-					    "to install it."));
+		d = ve_hig_dialog_new (NULL /* parent */,
+				       GTK_DIALOG_MODAL /* flags */,
+				       GTK_MESSAGE_ERROR,
+				       GTK_BUTTONS_OK,
+				       _("Xnest doesn't exist."),
+				       "%s",
+				       _("Please ask your system "
+					 "administrator "
+					 "to install it."));
 		gtk_widget_show_all (d);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (d);
@@ -482,18 +501,40 @@ main (int argc, char *argv[])
 		FILE *fp = NULL;
 		long pid;
 		
-		if ( ! xdmcp_enabled ||
-		     ! honor_indirect) {
+		if (( ! xdmcp_enabled ||
+		      ! honor_indirect) &&
+		    ! do_direct) {
 			GtkWidget *d;
-			d = gtk_message_dialog_new
+			d = ve_hig_dialog_new
 				(NULL /* parent */,
 				 GTK_DIALOG_MODAL /* flags */,
-				 GTK_MESSAGE_WARNING,
+				 GTK_MESSAGE_ERROR,
 				 GTK_BUTTONS_OK,
-				 _("Indirect XDMCP is not enabled,\n"
-				   "please ask your "
+				 _("Indirect XDMCP is not enabled"),
+				 "%s",
+				 _("Please ask your "
 				   "system administrator to enable "
-				   "it\nin the GDM configurator "
+				   "it in the GDM configurator "
+				   "program."));
+			gtk_widget_show_all (d);
+			gtk_dialog_run (GTK_DIALOG (d));
+			gtk_widget_destroy (d);
+			return 0;
+		}
+
+		if ( ! xdmcp_enabled &&
+		    do_direct) {
+			GtkWidget *d;
+			d = ve_hig_dialog_new
+				(NULL /* parent */,
+				 GTK_DIALOG_MODAL /* flags */,
+				 GTK_MESSAGE_ERROR,
+				 GTK_BUTTONS_OK,
+				 _("XDMCP is not enabled"),
+				 "%s",
+				 _("Please ask your "
+				   "system administrator to enable "
+				   "it in the GDM configurator "
 				   "program."));
 			gtk_widget_show_all (d);
 			gtk_dialog_run (GTK_DIALOG (d));
@@ -514,13 +555,14 @@ main (int argc, char *argv[])
 		    (kill (pid, 0) < 0 &&
 		     errno != EPERM)) {
 			GtkWidget *d;
-			d = gtk_message_dialog_new
+			d = ve_hig_dialog_new
 				(NULL /* parent */,
 				 GTK_DIALOG_MODAL /* flags */,
-				 GTK_MESSAGE_WARNING,
+				 GTK_MESSAGE_ERROR,
 				 GTK_BUTTONS_OK,
-				 _("GDM is not running.\n"
-				   "Please ask your "
+				 _("GDM is not running"),
+				 "%s",
+				 _("Please ask your "
 				   "system administrator to start it."));
 			gtk_widget_show_all (d);
 			gtk_dialog_run (GTK_DIALOG (d));
@@ -532,12 +574,13 @@ main (int argc, char *argv[])
 	display = get_free_display ();
 	if (display < 0) {
 		GtkWidget *d;
-		d = gtk_message_dialog_new (NULL /* parent */,
-					    GTK_DIALOG_MODAL /* flags */,
-					    GTK_MESSAGE_WARNING,
-					    GTK_BUTTONS_OK,
-					    _("Could not find a free "
-					      "display number"));
+		d = ve_hig_dialog_new (NULL /* parent */,
+				       GTK_DIALOG_MODAL /* flags */,
+				       GTK_MESSAGE_ERROR,
+				       GTK_BUTTONS_OK,
+				       _("Could not find a free "
+					 "display number"),
+				       /* avoid warning */ "%s", "");
 		gtk_widget_show_all (d);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (d);
