@@ -50,7 +50,7 @@ struct _Language {
  * The available languages come from the supplied locale.alias */
 static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
-	{ N_("A-M|Azerbaijani"), "az_AZ", NULL, 0 },
+	{ N_("A-M|Albanian"), "sq_AL", "Shqip", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Amharic"), "am_ET", NULL, 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
@@ -58,15 +58,25 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Arabic (Lebanon)"), "ar_LB", NULL, 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Azerbaijani"), "az_AZ", NULL, 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Basque"), "eu_ES", "Euskara", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Belarusian"), "be_BY", "Беларуская мова", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Bengali"), "bn_BD", "বাংলা", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Bengali (India)"), "bn_IN", "বাংলা", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Bulgarian"), "bg_BG", "Български", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Bosnian"), "bs_BA", NULL, 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Catalan"), "ca_ES", "Català", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Chinese (simplified)"), "zh_CN", "中文 (简体)", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
-	{ N_("A-M|Chinese (traditional)"), "zh_TW", "繁體中文", 0 },
+	{ N_("A-M|Chinese (traditional)"), "zh_TW", "中文 (繁體)", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Croatian"), "hr_HR", "Hrvatski", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
@@ -78,8 +88,6 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|English"), "en", "English", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
-	{ N_("A-M|Estonian"), "et_EE", "Eesti", 0 },
-	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|English (American)"), "en_US", "American English", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|English (Australian)"), "en_AU", "Australian English", 0 },
@@ -88,6 +96,8 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|English (Ireland)"), "en_IE", "Irish English", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Estonian"), "et_EE", "Eesti", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Finnish"), "fi_FI", "Suomi", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|French"), "fr_FR", "Français", 0 },
@@ -98,19 +108,30 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Greek"), "el_GR", "ελληνικά", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Gujarati"), "gu_IN", NULL, 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Hebrew"), "he_IL", "עברית", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Hebrew"), "iw_IL", "עברית", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Hindi"), "hi_IN", "বাংলা", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Hungarian"), "hu_HU", "Magyar", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Icelandic"), "is_IS", NULL, 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
-	{ N_("A-M|Irish"), "ga_IE", NULL, 0 },
+	{ N_("A-M|Indonesian"), "id_ID", "Bahasa Indonesia", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Interlingua"), "ia", NULL, 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Irish"), "ga_IE", "Gaeilge", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Italian"), "it_IT", "Italiano", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Japanese"), "ja_JP", "日本語", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Kannada"), "kn_IN", "﻿ಕನ್ನಡ", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Korean"), "ko_KR", "한국어", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Latvian"), "lv_LV", "Latviešu", 0 },
@@ -120,11 +141,23 @@ static Language languages [] = {
         { N_("A-M|Macedonian"), "mk_MK", NULL, 0 },
         /*Note translate the A-M to the A-M you used in the group label */
         { N_("A-M|Malay"), "ms_MY", "Bahasa Melayu", 0 },
+        /*Note translate the A-M to the A-M you used in the group label */
+        { N_("A-M|Malayalam"), "ml_IN", "മലയാളം", 0 },
+        /*Note translate the A-M to the A-M you used in the group label */
+        { N_("A-M|Marathi"), "mr_IN", NULL, 0 },
+        /*Note translate the A-M to the A-M you used in the group label */
+        { N_("A-M|Mongolian"), "mn_MN", "Монгол", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Norwegian (bokmal)"), "no_NO", "Norsk (bokmål)", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Norwegian (nynorsk)"), "nn_NO", "Norsk (nynorsk)", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Oriya"), "or_IN", NULL, 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Panjabi"), "pa_IN", NULL, 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Persian"), "fa_IR", "فارسی", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Polish"), "pl_PL", "Polski", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Portuguese"), "pt_PT", "Português", 0 },
@@ -135,6 +168,8 @@ static Language languages [] = {
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Russian"), "ru_RU", "русский", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Serbian"), "sr_YU", "", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Slovak"), "sk_SK", "Slovenský", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Slovenian"), "sl_SI", "Slovenščina", 0 },
@@ -149,6 +184,10 @@ static Language languages [] = {
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Tamil"), "ta_IN", NULL, 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Telugu"), "te_IN", NULL, 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Thai"), "th_TH", "ไทย", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Turkish"), "tr_TR", "Türkçe", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Ukrainian"), "uk_UA", "Українська", 0 },
@@ -156,6 +195,10 @@ static Language languages [] = {
 	{ N_("N-Z|Vietnamese"), "vi_VN", "Việt Nam", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Walloon"), "wa_BE", "Walon", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Welsh"), "cy_GB", "Cymraeg", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Yiddish"), "yi", "ייִדיש", 0 },
 	/* This is the POSIX/C locale for english, should really be in Other */
 	{ N_("Other|POSIX/C English"), "C", "POSIX/C English", 0 },
 	{ NULL, NULL }
