@@ -14,6 +14,7 @@
 #include "gdm.h"
 #include "gdmwm.h"
 #include "vicious.h"
+#include "viciousui.h"
 
 #include "greeter.h"
 #include "greeter_configuration.h"
@@ -22,6 +23,7 @@
 #include "greeter_item_clock.h"
 #include "greeter_item_pam.h"
 #include "greeter_item_ulist.h"
+#include "greeter_item_customlist.h"
 #include "greeter_item_capslock.h"
 #include "greeter_item_timed.h"
 #include "greeter_events.h"
@@ -39,10 +41,6 @@ gboolean GDM_IS_LOCAL = FALSE;
 char *GdmGraphicalTheme = NULL;
 char *GdmGraphicalThemeDir = NULL;
 int GdmXineramaScreen = 0;
-#if 0
-/* FIXME: maybe just whack this */
-gboolean GdmShowGnomeChooserSession = FALSE;
-#endif
 gboolean GdmShowGnomeFailsafeSession = FALSE;
 gboolean GdmShowXtermFailsafeSession = FALSE;
 gboolean GdmShowLastSession = FALSE;
@@ -70,6 +68,7 @@ int GdmMinimalUID;
 gboolean GdmAllowRoot;
 gboolean GdmAllowRemoteRoot;
 gchar *GdmWelcome;
+gchar *GdmServAuthDir;
 
 gboolean GdmUseCirclesInEntry = FALSE;
 
@@ -80,9 +79,12 @@ gint greeter_current_delay = 0;
 GreeterItemInfo *welcome_string_info = NULL;
 
 extern gboolean session_dir_whacked_out;
+extern gboolean require_quarter;
 
 gboolean greeter_probably_login_prompt = FALSE;
 
+static void gdm_login_abort (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
+
 static void 
 greeter_parse_config (void)
 {
@@ -123,10 +125,6 @@ greeter_parse_config (void)
 
     GdmShowXtermFailsafeSession = ve_config_get_bool (config, GDM_KEY_SHOW_XTERM_FAILSAFE);
     GdmShowGnomeFailsafeSession = ve_config_get_bool (config, GDM_KEY_SHOW_GNOME_FAILSAFE);
-#if 0
-    /* FIXME: maybe just whack this */
-    GdmShowGnomeChooserSession = ve_config_get_bool (config, GDM_KEY_SHOW_GNOME_CHOOSER);
-#endif
     GdmShowLastSession = ve_config_get_bool (config, GDM_KEY_SHOW_LAST_SESSION);
     GdmSessionDir = ve_config_get_string (config, GDM_KEY_SESSDIR);
     GdmDefaultSession = ve_config_get_string (config, GDM_KEY_DEFAULTSESSION);
@@ -139,6 +137,7 @@ greeter_parse_config (void)
     GdmSuspend = ve_config_get_string (config, GDM_KEY_SUSPEND);
     GdmConfigurator = ve_config_get_string (config, GDM_KEY_CONFIGURATOR);
     GdmGtkRC = ve_config_get_string (config, GDM_KEY_GTKRC);
+    GdmServAuthDir = ve_config_get_string (config, GDM_KEY_SERVAUTH);
 
     GdmWelcome = ve_config_get_translated_string (config, greeter_Welcome_key);
     /* A hack! */
@@ -200,8 +199,8 @@ greeter_parse_config (void)
     }
 }
 
-static void
-setup_cursor (GdkCursorType type)
+void
+greeter_setup_cursor (GdkCursorType type)
 {
 	GdkCursor *cursor = gdk_cursor_new (type);
 	gdk_window_set_cursor (gdk_get_default_root_window (), cursor);
@@ -263,10 +262,12 @@ greeter_ctrl_handler (GIOChannel *source,
 	buf[len-1] = '\0';
 
 	tmp = ve_locale_to_utf8 (buf);
-	if (tmp != NULL && strcmp (tmp, _("Username:")) == 0)
+	if (tmp != NULL && strcmp (tmp, _("Username:")) == 0) {
+		gdk_beep ();
 		greeter_probably_login_prompt = TRUE;
-	else
+	} else {
 		greeter_probably_login_prompt = FALSE;
+	}
 	greeter_item_pam_prompt (tmp, 128, TRUE);
 	g_free (tmp);
 	break;
@@ -311,14 +312,13 @@ greeter_ctrl_handler (GIOChannel *source,
 	gdm_wm_focus_new_windows (TRUE);
 
 	tmp = ve_locale_to_utf8 (buf);
-	dlg = gtk_message_dialog_new (NULL /* parent */,
-				      GTK_DIALOG_MODAL /* flags */,
-				      GTK_MESSAGE_ERROR,
-				      GTK_BUTTONS_OK,
-				      "%s",
-				      tmp);
+	dlg = ve_hig_dialog_new (NULL /* parent */,
+				 GTK_DIALOG_MODAL /* flags */,
+				 GTK_MESSAGE_ERROR,
+				 GTK_BUTTONS_OK,
+				 tmp,
+				 /* avoid warning */ "%s", "");
 	g_free (tmp);
-	gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 
 	gdm_wm_center_window (GTK_WINDOW (dlg));
 
@@ -402,6 +402,28 @@ greeter_ctrl_handler (GIOChannel *source,
         g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL); /* Empty */
 
 	greeter_item_timed_stop ();
+
+	if (require_quarter) {
+		/* we should be now fine for focusing new windows */
+		gdm_wm_focus_new_windows (TRUE);
+
+		dlg = ve_hig_dialog_new (NULL /* parent */,
+					 GTK_DIALOG_MODAL /* flags */,
+					 GTK_MESSAGE_INFO,
+					 GTK_BUTTONS_OK,
+					 /* translators:  This is a nice and evil eggie text, translate
+					  * to your favourite currency */
+					 _("Please insert 25 cents "
+					   "to log in."),
+					 /* avoid warning */ "%s", "");
+		gdm_wm_center_window (GTK_WINDOW (dlg));
+
+		gdm_wm_no_login_focus_push ();
+		gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_destroy (dlg);
+		gdm_wm_no_login_focus_pop ();
+	}
+
 	greeter_item_pam_leftover_messages ();
 
 	gdk_flush ();
@@ -413,47 +435,6 @@ greeter_ctrl_handler (GIOChannel *source,
 	_exit (EXIT_SUCCESS);
 	break;
 
-#if 0
-	/* FIXME: Maybe whack this */
-    case GDM_GNOMESESS:
-	{
-		char *sess;
-		GString *str = g_string_new (NULL);
-
-		do {
-			g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL);
-			buf[len-1] = '\0';
-			tmp = ve_locale_to_utf8 (buf);
-			g_string_append (str, tmp);
-			g_free (tmp);
-		} while (len == PIPE_SIZE-1);
-
-
-		sess = greeter_get_gnome_session (str->str);
-
-		g_string_free (str, TRUE);
-
-		tmp = ve_locale_from_utf8 (sess);
-		printf ("%c%s\n", STX, tmp);
-		fflush (stdout);
-		g_free (tmp);
-
-		g_free (sess);
-	}
-	break;
-
-    case GDM_SGNOMESESS:
-	g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL); /* Empty */
-
-	if (greeter_save_gnome_session ())
-	    printf ("%cY\n", STX);
-	else
-	    printf ("%c\n", STX);
-	fflush (stdout);
-
-	break;
-#endif
-
     case GDM_STARTTIMER:
 	g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL); /* Empty */
 
@@ -538,7 +519,7 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL); /* Empty */
 
 	/* Set busy cursor */
-	setup_cursor (GDK_WATCH);
+	greeter_setup_cursor (GDK_WATCH);
 
 	gdm_wm_save_wm_order ();
 
@@ -596,6 +577,9 @@ greeter_setup_items (void)
 					 NULL);
   greeter_item_system_setup ();
   greeter_item_session_setup ();
+
+  /* Setup the custom widgets */
+  greeter_item_customlist_setup ();
 }
 
 enum {
@@ -626,21 +610,21 @@ verify_gdm_version (void)
       gdm_wm_init (0);
       gdm_wm_focus_new_windows (TRUE);
     
-      dialog = gtk_message_dialog_new (NULL /* parent */,
-				       GTK_DIALOG_MODAL /* flags */,
-				       GTK_MESSAGE_ERROR,
-				       GTK_BUTTONS_OK,
-				       _("The greeter version (%s) does not match the daemon "
-					 "version.\n"
-					 "You have probably just upgraded gdm.\n"
-					 "Please restart the gdm daemon or reboot the computer."),
-				       VERSION);
-      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+      dialog = ve_hig_dialog_new (NULL /* parent */,
+				  GTK_DIALOG_MODAL /* flags */,
+				  GTK_MESSAGE_ERROR,
+				  GTK_BUTTONS_OK,
+				  _("Cannot start the greeter"),
+				  _("The greeter version (%s) does not match the daemon "
+				    "version.\n"
+				    "You have probably just upgraded gdm.\n"
+				    "Please restart the gdm daemon or reboot the computer."),
+				  VERSION);
     
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
 
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
     
       gtk_dialog_run (GTK_DIALOG (dialog));
     
@@ -656,16 +640,16 @@ verify_gdm_version (void)
       gdm_wm_init (0);
       gdm_wm_focus_new_windows (TRUE);
     
-      dialog = gtk_message_dialog_new (NULL /* parent */,
-				       GTK_DIALOG_MODAL /* flags */,
-				       GTK_MESSAGE_WARNING,
-				       GTK_BUTTONS_NONE,
-				       _("The greeter version (%s) does not match the daemon "
-					 "version.\n"
-					 "You have probably just upgraded gdm.\n"
-					 "Please restart the gdm daemon or reboot the computer."),
-				       VERSION);
-      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+      dialog = ve_hig_dialog_new (NULL /* parent */,
+				  GTK_DIALOG_MODAL /* flags */,
+				  GTK_MESSAGE_WARNING,
+				  GTK_BUTTONS_NONE,
+				  _("Cannot start the greeter"),
+				  _("The greeter version (%s) does not match the daemon "
+				    "version.\n"
+				    "You have probably just upgraded gdm.\n"
+				    "Please restart the gdm daemon or reboot the computer."),
+				  VERSION);
       gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 			      _("Reboot"),
 			      RESPONSE_REBOOT,
@@ -676,7 +660,7 @@ verify_gdm_version (void)
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
       
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
 
       switch (gtk_dialog_run (GTK_DIALOG (dialog)))
 	{
@@ -700,16 +684,16 @@ verify_gdm_version (void)
       gdm_wm_init (0);
       gdm_wm_focus_new_windows (TRUE);
     
-      dialog = gtk_message_dialog_new (NULL /* parent */,
-				       GTK_DIALOG_MODAL /* flags */,
-				       GTK_MESSAGE_WARNING,
-				       GTK_BUTTONS_NONE,
-				       _("The greeter version (%s) does not match the daemon "
-					 "version (%s).\n"
-					 "You have probably just upgraded gdm.\n"
-					 "Please restart the gdm daemon or reboot the computer."),
-				       VERSION, gdm_version);
-      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+      dialog = ve_hig_dialog_new (NULL /* parent */,
+				  GTK_DIALOG_MODAL /* flags */,
+				  GTK_MESSAGE_WARNING,
+				  GTK_BUTTONS_NONE,
+				  _("Cannot start the greeter"),
+				  _("The greeter version (%s) does not match the daemon "
+				    "version (%s).\n"
+				    "You have probably just upgraded gdm.\n"
+				    "Please restart the gdm daemon or reboot the computer."),
+				  VERSION, gdm_version);
       gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 			      _("Restart"),
 			      RESPONSE_RESTART,
@@ -725,7 +709,7 @@ verify_gdm_version (void)
     
       gtk_dialog_set_default_response (GTK_DIALOG (dialog), RESPONSE_RESTART);
       
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
 
       switch (gtk_dialog_run (GTK_DIALOG (dialog)))
 	{
@@ -752,16 +736,12 @@ greeter_message (const gchar *msg)
   /* we should be now fine for focusing new windows */
   gdm_wm_focus_new_windows (TRUE);
   
-  req = gtk_message_dialog_new (NULL /* parent */,
-				GTK_DIALOG_MODAL /* flags */,
-				GTK_MESSAGE_INFO,
-				GTK_BUTTONS_CLOSE,
-				"%s",
-				msg);
-  gtk_dialog_set_has_separator (GTK_DIALOG (req), FALSE);
-  g_signal_connect (G_OBJECT (req), "destroy",
-		    G_CALLBACK (gtk_widget_destroyed),
-		    &req);
+  req = ve_hig_dialog_new (NULL /* parent */,
+			   GTK_DIALOG_MODAL /* flags */,
+			   GTK_MESSAGE_INFO,
+			   GTK_BUTTONS_OK,
+			   msg,
+			   /* avoid warning */ "%s", "");
   
   gdm_wm_center_window (GTK_WINDOW (req));
   
@@ -769,33 +749,39 @@ greeter_message (const gchar *msg)
   gtk_dialog_run (GTK_DIALOG (req));
   gdm_wm_no_login_focus_pop ();
 
-  if (req)
-    gtk_widget_destroy (req);
+  gtk_widget_destroy (req);
 }
 
 
 gboolean
-greeter_query (const gchar *msg)
+greeter_query (const gchar *msg, const char *posbutton, const char *negbutton)
 {
 	int ret;
 	GtkWidget *req;
+	GtkWidget *button;
 
 	/* we should be now fine for focusing new windows */
 	gdm_wm_focus_new_windows (TRUE);
 
-	req = gtk_message_dialog_new (NULL /* parent */,
-				      GTK_DIALOG_MODAL /* flags */,
-				      GTK_MESSAGE_QUESTION,
-				      GTK_BUTTONS_YES_NO,
-				      "%s",
-				      msg);
-	gtk_dialog_set_has_separator (GTK_DIALOG (req), FALSE);
-	gtk_label_set_use_markup
-		(GTK_LABEL (GTK_MESSAGE_DIALOG (req)->label), TRUE);
+	req = ve_hig_dialog_new (NULL /* parent */,
+				 GTK_DIALOG_MODAL /* flags */,
+				 GTK_MESSAGE_QUESTION,
+				 GTK_BUTTONS_NONE,
+				 msg,
+				 /* avoid warning */ "%s", "");
 
-	g_signal_connect (G_OBJECT (req), "destroy",
-			  G_CALLBACK (gtk_widget_destroyed),
-			  &req);
+	button = gtk_button_new_from_stock (negbutton);
+	gtk_dialog_add_action_widget (GTK_DIALOG (req), button, GTK_RESPONSE_NO);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_show (button);
+
+	button = gtk_button_new_from_stock (posbutton);
+	gtk_dialog_add_action_widget (GTK_DIALOG (req), button, GTK_RESPONSE_YES);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_show (button);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (req),
+					 GTK_RESPONSE_YES);
 
 	gdm_wm_center_window (GTK_WINDOW (req));
 
@@ -803,8 +789,7 @@ greeter_query (const gchar *msg)
 	ret = gtk_dialog_run (GTK_DIALOG (req));
 	gdm_wm_no_login_focus_pop ();
 
-	if (req != NULL)
-	  gtk_widget_destroy (req);
+	gtk_widget_destroy (req);
 
 	if (ret == GTK_RESPONSE_YES)
 		return TRUE;
@@ -842,8 +827,6 @@ string_same (VeConfig *config, const char *cur, const char *key)
 		g_free (val);
 		return TRUE;
 	} else {
-		syslog (LOG_ERR, "string not same: cur '%s' val '%s' key '%s'",
-			cur, val, key);
 		g_free (val);
 		return FALSE;
 	}
@@ -856,8 +839,6 @@ bool_same (VeConfig *config, gboolean cur, const char *key)
 	if (ve_bool_equal (cur, val)) {
 		return TRUE;
 	} else {
-		syslog (LOG_ERR, "bool not same: cur '%d' val '%d' key '%s'",
-			cur, val, key);
 		return FALSE;
 	}
 }
@@ -869,8 +850,6 @@ int_same (VeConfig *config, int cur, const char *key)
 	if (cur == val) {
 		return TRUE;
 	} else {
-		syslog (LOG_ERR, "int not same: cur '%d' val '%d' key '%s'",
-			cur, val, key);
 		return FALSE;
 	}
 }
@@ -915,12 +894,6 @@ greeter_reread_config (int sig, gpointer data)
 	     ! bool_same (config,
 			  GdmShowGnomeFailsafeSession,
 			  GDM_KEY_SHOW_GNOME_FAILSAFE) ||
-#if 0
-	     /* FIXME: maybe just whack this */
-	     ! bool_same (config,
-			  GdmShowGnomeChooserSession,
-			  GDM_KEY_SHOW_GNOME_CHOOSER) ||
-#endif
 	     ! string_same (config,
 			    GdmSessionDir,
 			    GDM_KEY_SESSDIR) ||
@@ -954,11 +927,8 @@ greeter_reread_config (int sig, gpointer data)
 	     ! int_same (config,
 			 GdmTimedLoginDelay,
 			 GDM_KEY_TIMED_LOGIN_DELAY)) {
-		syslog (LOG_ERR, "something not same: "
-			"theme '%s' theme_dir '%s'",
-			theme, theme_dir);
 		/* Set busy cursor */
-		setup_cursor (GDK_WATCH);
+		greeter_setup_cursor (GDK_WATCH);
 
 		gdm_wm_save_wm_order ();
 
@@ -1114,6 +1084,28 @@ setup_background_color (void)
     }
 }
 
+static void
+gdm_login_abort (const gchar *format, ...)
+{
+    va_list args;
+    gchar *s;
+
+    if (!format) {
+	_exit (DISPLAY_GREETERFAILED);
+    }
+
+    va_start (args, format);
+    s = g_strdup_vprintf (format, args);
+    va_end (args);
+    
+    syslog (LOG_ERR, "%s", s);
+    closelog();
+
+    g_free (s);
+
+    _exit (DISPLAY_GREETERFAILED);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1143,7 +1135,7 @@ main (int argc, char *argv[])
   gtk_init (&argc, &argv);
 
   /* Should be a watch already, but anyway */
-  setup_cursor (GDK_WATCH);
+  greeter_setup_cursor (GDK_WATCH);
 
   if ( ! ve_string_empty (GdmGtkRC))
     gtk_rc_parse (GdmGtkRC);
@@ -1165,7 +1157,7 @@ main (int argc, char *argv[])
   sigaddset (&hup.sa_mask, SIGCHLD);
   
   if (sigaction (SIGHUP, &hup, NULL) < 0) 
-    g_error (_("main: Error setting up HUP signal handler"));
+    gdm_login_abort (_("%s: Error setting up %s signal handler: %s"), "main", "HUP", strerror (errno));
 
   term.sa_handler = greeter_done;
   term.sa_flags = 0;
@@ -1173,10 +1165,10 @@ main (int argc, char *argv[])
   sigaddset (&term.sa_mask, SIGCHLD);
   
   if (sigaction (SIGINT, &term, NULL) < 0) 
-    g_error (_("main: Error setting up INT signal handler"));
+    gdm_login_abort (_("%s: Error setting up %s signal handler: %s"), "main", "INT", strerror (errno));
   
   if (sigaction (SIGTERM, &term, NULL) < 0) 
-    g_error (_("main: Error setting up TERM signal handler"));
+    gdm_login_abort (_("%s: Error setting up %s signal handler: %s"), "main", "TERM", strerror (errno));
   
   sigfillset (&mask);
   sigdelset (&mask, SIGTERM);
@@ -1185,7 +1177,7 @@ main (int argc, char *argv[])
   sigdelset (&mask, SIGCHLD);
   
   if (sigprocmask (SIG_SETMASK, &mask, NULL) == -1) 
-    g_error (_("Could not set signal mask!"));
+    gdm_login_abort (_("Could not set signal mask!"));
 
   
   if (! DOING_GDM_DEVELOPMENT) {
@@ -1243,29 +1235,35 @@ main (int argc, char *argv[])
 	DOING_GDM_DEVELOPMENT)
       {
         GtkWidget *dialog;
+	char *s;
 
         gdm_wm_init (0);
         gdm_wm_focus_new_windows (TRUE);
     
-        dialog = gtk_message_dialog_new (NULL /* parent */,
-				         GTK_DIALOG_MODAL /* flags */,
-				         GTK_MESSAGE_ERROR,
-				         GTK_BUTTONS_OK,
-				         _("There was an error loading the "
-					   "theme %s"),
-					 g_getenv ("GDM_THEME"));
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+	s = g_strdup_printf (_("There was an error loading the "
+			       "theme %s"),
+			     g_getenv ("GDM_THEME"));
+        dialog = ve_hig_dialog_new (NULL /* parent */,
+				    GTK_DIALOG_MODAL /* flags */,
+				    GTK_MESSAGE_ERROR,
+				    GTK_BUTTONS_OK,
+				    s,
+				    "%s", (error && error->message) ? error->message : "");
+	g_free (s);
     
         gtk_widget_show_all (dialog);
         gdm_wm_center_window (GTK_WINDOW (dialog));
 
-        setup_cursor (GDK_LEFT_PTR);
+        greeter_setup_cursor (GDK_LEFT_PTR);
     
         gtk_dialog_run (GTK_DIALOG (dialog));
 
         exit(1);
       }
 
+  if (error)
+    g_clear_error (&error);
+
   /* Try circles.xml */
   if (root == NULL)
     {
@@ -1279,27 +1277,28 @@ main (int argc, char *argv[])
 			    NULL);
     }
 
-  if (greeter_lookup_id ("user-pw-entry") == NULL)
+  if (root != NULL && greeter_lookup_id ("user-pw-entry") == NULL)
     {
       GtkWidget *dialog;
 
       gdm_wm_init (0);
       gdm_wm_focus_new_windows (TRUE);
     
-      dialog = gtk_message_dialog_new (NULL /* parent */,
-				       GTK_DIALOG_MODAL /* flags */,
-				       GTK_MESSAGE_ERROR,
-				       GTK_BUTTONS_OK,
-				       _("The theme for the graphical greeter "
-					 "is corrupt.  It does not contain "
-					 "definition for the username/password "
-					 "entry element."));
-      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+      dialog = ve_hig_dialog_new (NULL /* parent */,
+				  GTK_DIALOG_MODAL /* flags */,
+				  GTK_MESSAGE_ERROR,
+				  GTK_BUTTONS_OK,
+				  _("The theme for the graphical greeter "
+				    "is corrupt"),
+				  "%s",
+				  _("The theme does not contain "
+				    "definition for the username/password "
+				    "entry element."));
 
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
 
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
 
       gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -1320,42 +1319,42 @@ main (int argc, char *argv[])
       gdm_wm_init (0);
       gdm_wm_focus_new_windows (TRUE);
     
-      dialog = gtk_message_dialog_new (NULL /* parent */,
-				       GTK_DIALOG_MODAL /* flags */,
-				       GTK_MESSAGE_ERROR,
-				       GTK_BUTTONS_OK,
-				       _("There was an error loading the "
-					 "theme, and the default theme "
-					 "also could not have been loaded, "
-					 "I will attempt to start the "
-					 "standard greeter"));
-      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+      dialog = ve_hig_dialog_new (NULL /* parent */,
+				  GTK_DIALOG_MODAL /* flags */,
+				  GTK_MESSAGE_ERROR,
+				  GTK_BUTTONS_OK,
+				  _("There was an error loading the "
+				    "theme, and the default theme "
+				    "also could not have been loaded, "
+				    "I will attempt to start the "
+				    "standard greeter"),
+				  /* avoid warning */ "%s", "");
     
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
 
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
     
       gtk_dialog_run (GTK_DIALOG (dialog));
 
       execl (EXPANDED_BINDIR "/gdmlogin", EXPANDED_BINDIR "/gdmlogin", NULL);
       execlp ("gdmlogin", "gdmlogin", NULL);
 
-      dialog = gtk_message_dialog_new (NULL /* parent */,
-				       GTK_DIALOG_MODAL /* flags */,
-				       GTK_MESSAGE_ERROR,
-				       GTK_BUTTONS_OK,
-				       _("I could not start the standard "
-					 "greeter.  This display will abort "
-					 "and you may have to login another "
-					 "way and fix the installation of "
-					 "gdm"));
-      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+      dialog = ve_hig_dialog_new (NULL /* parent */,
+				  GTK_DIALOG_MODAL /* flags */,
+				  GTK_MESSAGE_ERROR,
+				  GTK_BUTTONS_OK,
+				  _("I could not start the standard "
+				    "greeter.  This display will abort "
+				    "and you may have to login another "
+				    "way and fix the installation of "
+				    "gdm"),
+				  /* avoid warning */ "%s", "");
     
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
 
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
     
       gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -1389,18 +1388,19 @@ main (int argc, char *argv[])
 
       gdm_wm_focus_new_windows (TRUE);
 
-      dialog = gtk_message_dialog_new (NULL /* parent */,
-				       GTK_DIALOG_MODAL /* flags */,
-				       GTK_MESSAGE_ERROR,
-				       GTK_BUTTONS_OK,
-				       _("Your session directory is missing or empty!\n\n"
-					 "There are two available sessions you can use, but\n"
-					 "you should log in and correct the gdm configuration."));
-      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+      dialog = ve_hig_dialog_new (NULL /* parent */,
+				  GTK_DIALOG_MODAL /* flags */,
+				  GTK_MESSAGE_ERROR,
+				  GTK_BUTTONS_OK,
+				  _("Session directory is missing"),
+				  "%s",
+				  _("Your session directory is missing or empty!  "
+				    "There are two available sessions you can use, but "
+				    "you should log in and correct the gdm configuration."));
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
 
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
 
       gdm_wm_no_login_focus_push ();
       gtk_dialog_run (GTK_DIALOG (dialog));
@@ -1414,18 +1414,20 @@ main (int argc, char *argv[])
 
       gdm_wm_focus_new_windows (TRUE);
 
-      dialog = gtk_message_dialog_new (NULL /* parent */,
-				       GTK_DIALOG_MODAL /* flags */,
-				       GTK_MESSAGE_ERROR,
-				       GTK_BUTTONS_OK,
-				       _("The configuration file contains an invalid command\n"
-					 "line for the login dialog, and thus I ran the\n"
-					 "default command.  Please fix your configuration."));
-      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+      dialog = ve_hig_dialog_new (NULL /* parent */,
+				  GTK_DIALOG_MODAL /* flags */,
+				  GTK_MESSAGE_ERROR,
+				  GTK_BUTTONS_OK,
+				  _("Configuration is not correct"),
+				  "%s",
+				  _("The configuration file contains an invalid command "
+				    "line for the login dialog, and thus I ran the "
+				    "default command.  Please fix your configuration."));
+
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
 
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
 
       gdm_wm_no_login_focus_push ();
       gtk_dialog_run (GTK_DIALOG (dialog));
@@ -1440,19 +1442,21 @@ main (int argc, char *argv[])
 
       gdm_wm_focus_new_windows (TRUE);
 
-      dialog = gtk_message_dialog_new (NULL /* parent */,
-				       GTK_DIALOG_MODAL /* flags */,
-				       GTK_MESSAGE_ERROR,
-				       GTK_BUTTONS_OK,
-				       _("The configuration was not found.  GDM is using\n"
-					 "defaults to run this session.  You should log in\n"
-					 "and create a configuration file with the GDM\n"
-					 "configuration program."));
-      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+      dialog = ve_hig_dialog_new (NULL /* parent */,
+				  GTK_DIALOG_MODAL /* flags */,
+				  GTK_MESSAGE_ERROR,
+				  GTK_BUTTONS_OK,
+				  _("No configuration was found"),
+				  "%s",
+				  _("The configuration was not found.  GDM is using "
+				    "defaults to run this session.  You should log in "
+				    "and create a configuration file with the GDM "
+				    "configuration program."));
+
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
 
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
 
       gdm_wm_no_login_focus_push ();
       gtk_dialog_run (GTK_DIALOG (dialog));
@@ -1462,7 +1466,7 @@ main (int argc, char *argv[])
 
   gdm_wm_restore_wm_order ();
 
-  setup_cursor (GDK_LEFT_PTR);
+  greeter_setup_cursor (GDK_LEFT_PTR);
 
   gtk_main ();
 
